@@ -53,8 +53,6 @@
 #include "gdm-session-record.h"
 #include "gdm-session-worker-job.h"
 
-#include "ck-connector.h"
-
 #define GDM_SESSION_DBUS_PATH         "/org/gnome/DisplayManager/Session"
 #define GDM_SESSION_DBUS_INTERFACE    "org.gnome.DisplayManager.Session"
 #define GDM_SESSION_DBUS_ERROR_CANCEL "org.gnome.DisplayManager.Session.Error.Cancel"
@@ -70,12 +68,13 @@ struct _GdmSessionDirectPrivate
         char                *saved_session;
         char                *selected_language;
         char                *saved_language;
+        char                *selected_layout;
+        char                *saved_layout;
         char                *selected_user;
         char                *user_x11_authority_file;
 
         DBusMessage         *message_pending_reply;
         DBusConnection      *worker_connection;
-        CkConnector         *ckc;
 
         GdmSessionWorkerJob *job;
         GPid                 session_pid;
@@ -596,6 +595,16 @@ get_default_language_name (GdmSessionDirect *session)
     return setlocale (LC_MESSAGES, NULL);
 }
 
+static const char *
+get_default_layout_name (GdmSessionDirect *session)
+{
+    if (session->priv->saved_layout != NULL) {
+                return session->priv->saved_layout;
+    }
+
+    return "us";
+}
+
 static char *
 get_fallback_session_name (void)
 {
@@ -661,6 +670,8 @@ gdm_session_direct_defaults_changed (GdmSessionDirect *session)
 {
         _gdm_session_default_language_name_changed (GDM_SESSION (session),
                                                     get_default_language_name (session));
+        _gdm_session_default_layout_name_changed (GDM_SESSION (session),
+                                                  get_default_layout_name (session));
         _gdm_session_default_session_name_changed (GDM_SESSION (session),
                                                    get_default_session_name (session));
 }
@@ -681,6 +692,9 @@ gdm_session_direct_select_user (GdmSession *session,
 
         g_free (impl->priv->saved_language);
         impl->priv->saved_language = NULL;
+
+        g_free (impl->priv->saved_layout);
+        impl->priv->saved_layout = NULL;
 }
 
 static DBusHandlerResult
@@ -1034,6 +1048,39 @@ gdm_session_direct_handle_saved_language_name_read (GdmSessionDirect *session,
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static DBusHandlerResult
+gdm_session_direct_handle_saved_layout_name_read (GdmSessionDirect *session,
+                                                  DBusConnection   *connection,
+                                                  DBusMessage      *message)
+{
+        DBusMessage *reply;
+        DBusError    error;
+        const char  *layout_name;
+
+        dbus_error_init (&error);
+        if (! dbus_message_get_args (message, &error,
+                                     DBUS_TYPE_STRING, &layout_name,
+                                     DBUS_TYPE_INVALID)) {
+                g_warning ("ERROR: %s", error.message);
+        }
+
+        reply = dbus_message_new_method_return (message);
+        dbus_connection_send (connection, reply, NULL);
+        dbus_message_unref (reply);
+
+        if (strcmp (layout_name,
+                    get_default_layout_name (session)) != 0) {
+                g_free (session->priv->saved_layout);
+                session->priv->saved_layout = g_strdup (layout_name);
+
+                _gdm_session_default_layout_name_changed (GDM_SESSION (session),
+                                                          layout_name);
+        }
+
+
+        return DBUS_HANDLER_RESULT_HANDLED;
+}
+
 static DBusHandlerResult
 gdm_session_direct_handle_saved_session_name_read (GdmSessionDirect *session,
                                                    DBusConnection   *connection,
@@ -1135,6 +1182,8 @@ session_worker_message (DBusConnection *connection,
                 return gdm_session_direct_handle_session_died (session, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "SavedLanguageNameRead")) {
                 return gdm_session_direct_handle_saved_language_name_read (session, connection, message);
+        } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "SavedLayoutNameRead")) {
+                return gdm_session_direct_handle_saved_layout_name_read (session, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_SESSION_DBUS_INTERFACE, "SavedSessionNameRead")) {
                 return gdm_session_direct_handle_saved_session_name_read (session, connection, message);
         }
@@ -1580,6 +1629,19 @@ start_worker (GdmSessionDirect *session)
 static void
 stop_worker (GdmSessionDirect *session)
 {
+        g_signal_handlers_disconnect_by_func (session->priv->job,
+                                              G_CALLBACK (worker_stopped),
+                                              session);
+        g_signal_handlers_disconnect_by_func (session->priv->job,
+                                              G_CALLBACK (worker_started),
+                                              session);
+        g_signal_handlers_disconnect_by_func (session->priv->job,
+                                              G_CALLBACK (worker_exited),
+                                              session);
+        g_signal_handlers_disconnect_by_func (session->priv->job,
+                                              G_CALLBACK (worker_died),
+                                              session);
+
         cancel_pending_query (session);
 
         if (session->priv->worker_connection != NULL) {
@@ -1588,6 +1650,7 @@ stop_worker (GdmSessionDirect *session)
         }
 
         gdm_session_worker_job_stop (session->priv->job);
+        g_object_unref (session->priv->job);
         session->priv->job = NULL;
 }
 
@@ -1833,6 +1896,16 @@ get_language_name (GdmSessionDirect *session)
         return get_default_language_name (session);
 }
 
+static const char *
+get_layout_name (GdmSessionDirect *session)
+{
+        if (session->priv->selected_layout != NULL) {
+                return session->priv->selected_layout;
+        }
+
+        return get_default_layout_name (session);
+}
+
 static const char *
 get_session_name (GdmSessionDirect *session)
 {
@@ -1866,73 +1939,6 @@ get_session_command (GdmSessionDirect *session)
         return command;
 }
 
-static gboolean
-open_ck_session (GdmSessionDirect *session)
-{
-        struct passwd *pwent;
-        gboolean       ret;
-        int            res;
-        DBusError      error;
-        const char     *display_name;
-        const char     *display_device;
-        const char     *display_hostname;
-
-        if (session->priv->display_name != NULL) {
-                display_name = session->priv->display_name;
-        } else {
-                display_name = "";
-        }
-        if (session->priv->display_hostname != NULL) {
-                display_hostname = session->priv->display_hostname;
-        } else {
-                display_hostname = "";
-        }
-        if (session->priv->display_device != NULL) {
-                display_device = session->priv->display_device;
-        } else {
-                display_device = "";
-        }
-
-        g_assert (session->priv->selected_user != NULL);
-
-        pwent = getpwnam (session->priv->selected_user);
-        if (pwent == NULL) {
-                return FALSE;
-        }
-
-        session->priv->ckc = ck_connector_new ();
-        if (session->priv->ckc == NULL) {
-                g_warning ("Couldn't create new ConsoleKit connector");
-                goto out;
-        }
-
-        dbus_error_init (&error);
-        res = ck_connector_open_session_with_parameters (session->priv->ckc,
-                                                         &error,
-                                                         "unix-user", &pwent->pw_uid,
-                                                         "x11-display", &display_name,
-                                                         "x11-display-device", &display_device,
-                                                         "remote-host-name", &display_hostname,
-                                                         "is-local", &session->priv->display_is_local,
-                                                         NULL);
-
-        if (! res) {
-                if (dbus_error_is_set (&error)) {
-                        g_warning ("%s\n", error.message);
-                        dbus_error_free (&error);
-                } else {
-                        g_warning ("cannot open CK session: OOM, D-Bus system bus not available,\n"
-                                   "ConsoleKit not available or insufficient privileges.\n");
-                }
-                goto out;
-        }
-
-        ret = TRUE;
-
- out:
-        return ret;
-}
-
 static void
 gdm_session_direct_set_environment_variable (GdmSessionDirect *session,
                                              const char       *key,
@@ -1951,15 +1957,6 @@ gdm_session_direct_set_environment_variable (GdmSessionDirect *session,
 static void
 setup_session_environment (GdmSessionDirect *session)
 {
-        const char *session_cookie;
-        gboolean    res;
-
-        session_cookie = NULL;
-        res = open_ck_session (session);
-        if (res) {
-                session_cookie = ck_connector_get_cookie (session->priv->ckc);
-        }
-
         gdm_session_direct_set_environment_variable (session,
                                                      "GDMSESSION",
                                                      get_session_name (session));
@@ -1974,14 +1971,13 @@ setup_session_environment (GdmSessionDirect *session)
                                                      "GDM_LANG",
                                                      get_language_name (session));
 
+        gdm_session_direct_set_environment_variable (session,
+                                                     "GDM_KEYBOARD_LAYOUT",
+                                                     get_layout_name (session));
+
         gdm_session_direct_set_environment_variable (session,
                                                      "DISPLAY",
                                                      session->priv->display_name);
-        if (session_cookie != NULL) {
-                gdm_session_direct_set_environment_variable (session,
-                                                             "XDG_SESSION_COOKIE",
-                                                             session_cookie);
-        }
 
         if (session->priv->user_x11_authority_file != NULL) {
                 gdm_session_direct_set_environment_variable (session,
@@ -2030,12 +2026,6 @@ gdm_session_direct_close (GdmSession *session)
 
         g_debug ("GdmSessionDirect: Closing session");
 
-        if (impl->priv->ckc != NULL) {
-                ck_connector_close_session (impl->priv->ckc, NULL);
-                ck_connector_unref (impl->priv->ckc);
-                impl->priv->ckc = NULL;
-        }
-
         if (impl->priv->job != NULL) {
                 if (impl->priv->is_running) {
                         gdm_session_record_logout (impl->priv->session_pid,
@@ -2063,6 +2053,12 @@ gdm_session_direct_close (GdmSession *session)
         g_free (impl->priv->saved_language);
         impl->priv->saved_language = NULL;
 
+        g_free (impl->priv->selected_layout);
+        impl->priv->selected_layout = NULL;
+
+        g_free (impl->priv->saved_layout);
+        impl->priv->saved_layout = NULL;
+
         g_free (impl->priv->user_x11_authority_file);
         impl->priv->user_x11_authority_file = NULL;
 
@@ -2138,6 +2134,24 @@ gdm_session_direct_select_language (GdmSession *session,
                                  get_language_name (impl));
 }
 
+static void
+gdm_session_direct_select_layout (GdmSession *session,
+                                  const char *text)
+{
+        GdmSessionDirect *impl = GDM_SESSION_DIRECT (session);
+
+        g_free (impl->priv->selected_layout);
+
+        if (strcmp (text, "__previous") == 0) {
+                impl->priv->selected_layout = NULL;
+        } else {
+                impl->priv->selected_layout = g_strdup (text);
+        }
+
+        send_dbus_string_signal (impl, "SetLayoutName",
+                                 get_layout_name (impl));
+}
+
 static void
 _gdm_session_direct_set_display_id (GdmSessionDirect *session,
                                     const char       *id)
@@ -2330,6 +2344,8 @@ gdm_session_direct_finalize (GObject *object)
         g_free (session->priv->saved_session);
         g_free (session->priv->selected_language);
         g_free (session->priv->saved_language);
+        g_free (session->priv->selected_layout);
+        g_free (session->priv->saved_layout);
 
         parent_class = G_OBJECT_CLASS (gdm_session_direct_parent_class);
 
@@ -2405,6 +2421,7 @@ gdm_session_iface_init (GdmSessionIface *iface)
         iface->answer_query = gdm_session_direct_answer_query;
         iface->select_session = gdm_session_direct_select_session;
         iface->select_language = gdm_session_direct_select_language;
+        iface->select_layout = gdm_session_direct_select_layout;
         iface->select_user = gdm_session_direct_select_user;
 }
 
